﻿'DI_149.ChannelArray map:
'ChannelArray(0-7) analog inputs 0-7
'ChannelArray(8) all digital inputs (all four bits)
'ChannelArray(9) frequency input
'ChannelArray(10) count input
'ChannelArray(11) digital outputs (all four bits)

Imports dataq.simple

Public Class frmDI_149Example
    Dim DataqDeviceArray() As DataqDevice 'array containing all connected devices
    Dim WithEvents DI_149 As DataqDevice 'this is the primary device we'll work with
    Dim CollectedData(11) As Double 'dimension the array that will hold acquired data
    Dim ChannelConfig(11) As String 'array to retain the channel configuration defined via the GUI

    Private Sub frmDI_149Example_Load(sender As System.Object, e As System.EventArgs) Handles MyBase.Load
        'upon loading the application do some device discovery, connection, and housekeeping

        'Discover connected devices
        DataqDeviceArray = Discovery.DiscoverAllDevices

        'Use only the first device in the discovery array
        'Bail if the first arrary position is empty
        If DataqDeviceArray.Length = 0 Then
            MessageBox.Show("No compatible DATAQ Instruments devices found.", _
                            "Missing DATAQ Instruments hardware", _
                            MessageBoxButtons.OK, _
                            MessageBoxIcon.Error)
            End

            'get here if the first array position contains something
        Else
            DI_149 = DataqDeviceArray(0) 'assign DataqDevice DI_149 to the first device in the array and...
            DI_149.Connect() '...connect to it
        End If

        'display model-specific information
        lblModel.Text = DI_149.Model
        lblSerial.Text = DI_149.SerialNumber
        lblPort.Text = DI_149.ComPort

        'Bail if the connected device is not model DI-149
        If DI_149.Model <> "DI-149" Then
            MessageBox.Show("This program works only with DATAQ Instruments model DI-149", _
                           "Incompatible Device (" + DI_149.Model + ")", _
                           MessageBoxButtons.OK, _
                           MessageBoxIcon.Error)
            End
        End If

        'load up ChannelConfig list with a default analog channel configuration
        For Channel As Integer = 0 To 7
            ChannelConfig(Channel) = "On" 'Enable the all analog input channels
        Next
        'rbAin1 is the only one checked upon load so...
        cbAnalogConfig.SelectedIndex = 1 'Display "On" to be consistent

        'now define the default settings for the digital inputs
        ChannelConfig(8) = "Off" 'turn off all discrete bits
        ChannelConfig(9) = "Off" ' turn off rate
        ChannelConfig(10) = "Off" ' turn off count


        'display "Off" to be consistent for all digital inputs
        cbDin0_3.SelectedIndex = 0
        cbRate.SelectedIndex = 0
        cbCount.SelectedIndex = 0

        'default is to configure an analog channel so hide the digital config combo boxes
        cbDin0_3.Visible = False
        cbRate.Visible = False
        cbCount.Visible = False

        'everthing's good, so create an event handler that fires when new DI-149 data is available
        AddHandler DI_149.NewData, AddressOf GetDI149Data
    End Sub
    Private Sub GetDI149Data()
        'this is the event handler for DI-149 data. Get here when new data is available during a scan
        Dim Scans As Integer = DI_149.NumberOfScansAvailable
        Dim Channels As Short = DI_149.NumberOfChannelsEnabled
        Dim DI_149_Data(Scans * Channels - 1) As Double 'will hold all data for the scan
        Dim ResponseString As String = ""

        'Move data into temporary array
        DI_149.GetInterleavedScaledData(DI_149_Data, 0, Scans)
        'now move it to the list box
        For Row As Integer = 0 To Scans - 1
            For Column As Short = 0 To Channels - 1
                ResponseString = ResponseString + DI_149_Data(Column + (Row * Channels)).ToString("'+'00.00;'-'00.00")
                If Column <> Channels - 1 Then
                    ResponseString = ResponseString + ", " 'insert a comma only if this is not the last displayed value
                End If
            Next
            UpdateList(ResponseString + vbCrLf) 'output to the list box
            ResponseString = "" 'reset ResponseString
        Next
    End Sub
    Private Sub btnStart_Click(sender As System.Object, e As System.EventArgs) Handles btnStart.Click
        'this sub configures the DI-149 scan list. 
        'DI_149.ChannelArray map:
        'ChannelArray(0-7) analog inputs 0-7 (or 1-8 for normal people as per the device sticker)
        'ChannelArray(8) all digital inputs (all four bits)
        'ChannelArray(9) frequency input
        'ChannelArray(10) count input

        Dim Range As New dataq.Range(Of Double) 'holds the FSR of the rate channel. Analog ranges are fixed at ±10 Vfs

        'first, some GUI housekeeping
        btnStart.Enabled = False
        btnStop.Enabled = True
        cbSampleRate.Enabled = False
        lbOutput.Items.Clear()
        lblSamplingRate.Text = ""
        lblRealTime.Text = ""
        cbAnalogConfig.Enabled = False
        cbDin0_3.Enabled = False
        cbRate.Enabled = False
        cbCount.Enabled = False
        rbAin1.Enabled = False
        rbAin2.Enabled = False
        rbAin3.Enabled = False
        rbAin4.Enabled = False
        rbAin5.Enabled = False
        rbAin6.Enabled = False
        rbAin7.Enabled = False
        rbAin8.Enabled = False
        rbDin0_1.Enabled = False
        rbRate.Enabled = False
        rbCount.Enabled = False

        'Configure the DI-149's scan list for analog channels
        'do the following for each of the eight analog channels:
        For Channel As Integer = 0 To 7
            'first disable the channel
            DI_149.ChannelArray(Channel).Enabled = False

            'now configure the channel as defined by array ChannelConfig
            If ChannelConfig(Channel) <> "Off" Then
                'not OFF so must be on
                DI_149.ChannelArray(Channel).Enabled = True ' enable the channel for acq
            End If
        Next

        'now set up the digital input channels

        For Channel As Integer = 8 To 10
            DI_149.ChannelArray(Channel).Enabled = False 'disable the channel first to aid management
            Select Case True
                Case Channel = 8
                    'this handles the acquisition of all digital input bit states (0-3)
                    If ChannelConfig(Channel) <> "Off" Then
                        'not OFF so must be on, so enable 
                        DI_149.ChannelArray(Channel).Enabled = True
                    End If
                Case Channel = 9
                    'this handles the rate input bit (Din2)
                    If ChannelConfig(Channel) <> "Off" Then
                        'not OFF so must be on
                        Range.Maximum = ChannelConfig(Channel) 'define the Range variable...
                        DI_149.ChannelArray(Channel).InputRange = Range '...and set it
                        DI_149.ChannelArray(Channel).Enabled = True ' finally, enable the channel for acq
                    End If
                Case Channel = 10
                    'this handles the count input bit (Din3)
                    If ChannelConfig(Channel) <> "Off" Then
                        'not OFF so must be on, so enable it
                        DI_149.ChannelArray(Channel).Enabled = True
                    End If

            End Select
        Next

        'set the sampling rate
        'Note: the class will always select the sampling rate closest to the one selected constrained
        'by the number of enabled channels. For a full treatment of how to configure DI-149 sampling rate
        'refer to the instrument's prototocol document:
        'http://www.dataq.com/resources/pdfs/misc/di-149-protocol.pdf
        DI_149.SampleRatePerChannel = Val(cbSampleRate.Text)

        'display the actual sampling rate per channel
        lblSamplingRate.Text = DI_149.SampleRatePerChannel.ToString("F3")

        DI_149.NewDataMinimum = 1 'set the number of scans to acquire before the NewData event fires
        DI_149.Start() 'start scanning
        SetDigitalOutputs() 'set defined states of the digital output port
    End Sub
    Private Sub SetDigitalOutputs()
        'Construct a value between 0-15 (4-bit) to write to the digital output port as a function of
        'checked (enabled) bits. When a bit is checked the output is driven low (each is a low-true output)
        Dim DigitalOutValue As Short = 0
        For Port As Short = 0 To 3
            Select Case True
                Case (Port = 0) And ckbDout0.Checked = True
                    DigitalOutValue = DigitalOutValue + 2 ^ Port
                Case (Port = 1) And ckbDout1.Checked = True
                    DigitalOutValue = DigitalOutValue + 2 ^ Port
                Case (Port = 2) And ckbDout2.Checked = True
                    DigitalOutValue = DigitalOutValue + 2 ^ Port
                Case (Port = 3) And ckbDout3.Checked = True
                    DigitalOutValue = DigitalOutValue + 2 ^ Port
            End Select
        Next
        DI_149.ChannelArray(11).Write(DigitalOutValue) 'write the value to the ports asynchronously during data acq
        'use the asynchrounous feature for real time alarms and such. 
    End Sub
    Private Sub btnStop_Click(sender As System.Object, e As System.EventArgs) Handles btnStop.Click
        'stop button clicked so stop the DI-149 and perform some GUI housekeeping
        DI_149.Stop()
        'gui housekeeping stuff
        btnStart.Enabled = True
        btnStop.Enabled = False
        cbSampleRate.Enabled = True
        cbAnalogConfig.Enabled = False
        cbDin0_3.Enabled = False
        cbRate.Enabled = False
        cbCount.Enabled = False

        cbAnalogConfig.Enabled = True
        cbDin0_3.Enabled = True
        cbRate.Enabled = True
        cbCount.Enabled = True
        rbAin1.Enabled = True
        rbAin2.Enabled = True
        rbAin3.Enabled = True
        rbAin4.Enabled = True
        rbAin5.Enabled = True
        rbAin6.Enabled = True
        rbAin7.Enabled = True
        rbAin8.Enabled = True
        rbDin0_1.Enabled = True
        rbRate.Enabled = True
        rbCount.Enabled = True
    End Sub
    'UpdateList is called from a different thread than the main program, so we need to delegate
    Private Delegate Sub UpdateListDelegate(ByVal ResponseString As String) 'create new class UpdateListDelegate
    Private Sub UpdateList(ByVal ResponseString As String)
        If Me.InvokeRequired Then
            Me.Invoke(New UpdateListDelegate(AddressOf UpdateList), ResponseString) 'invoke delegate then...
        Else
            lbOutput.Items.Add(ResponseString) '...output string to list box
            lblRealTime.Text = ResponseString
        End If
    End Sub
    Private Sub rbResetCount_Click(sender As System.Object, e As System.EventArgs) Handles rbResetCount.Click
        DI_149.ChannelArray(10).Reset()
    End Sub
    'the below is all mundane GUI management stuff, none of which accesses the DI-149 directly. 
    Private Sub cbChannelConfig_SelectedIndexChanged(sender As System.Object, e As System.EventArgs) Handles cbAnalogConfig.SelectedIndexChanged
        'When combobox cbChannelConfig changes, update array ChannelConfig accordingly
        'as a function of the enabled channel
        Select Case True
            Case rbAin1.Checked
                ChannelConfig(0) = cbAnalogConfig.Text
            Case rbAin2.Checked
                ChannelConfig(1) = cbAnalogConfig.Text
            Case rbAin3.Checked
                ChannelConfig(2) = cbAnalogConfig.Text
            Case rbAin4.Checked
                ChannelConfig(3) = cbAnalogConfig.Text
            Case rbAin5.Checked
                ChannelConfig(4) = cbAnalogConfig.Text
            Case rbAin6.Checked
                ChannelConfig(5) = cbAnalogConfig.Text
            Case rbAin7.Checked
                ChannelConfig(6) = cbAnalogConfig.Text
            Case rbAin8.Checked
                ChannelConfig(7) = cbAnalogConfig.Text
        End Select
    End Sub

    Private Sub rbAin1_CheckedChanged(sender As System.Object, e As System.EventArgs) Handles rbAin1.CheckedChanged
        'display the proper combo box
        cbAnalogConfig.Visible = True
        cbDin0_3.Visible = False
        cbRate.Visible = False
        cbCount.Visible = False
        'diplay the configuration of analog channel 1 in combo box cbChannelConfig
        If rbAin1.Checked Then
            cbAnalogConfig.Text = ChannelConfig(0)
        End If
    End Sub

    Private Sub rbAin2_CheckedChanged(sender As System.Object, e As System.EventArgs) Handles rbAin2.CheckedChanged
        'display the proper combo box
        cbAnalogConfig.Visible = True
        cbDin0_3.Visible = False
        cbRate.Visible = False
        cbCount.Visible = False
        'diplay the configuration of analog channel 2 in combo box cbChannelConfig
        If rbAin2.Checked Then
            cbAnalogConfig.Text = ChannelConfig(1)
        End If
    End Sub

    Private Sub rbAin3_CheckedChanged(sender As System.Object, e As System.EventArgs) Handles rbAin3.CheckedChanged
        'display the proper combo box
        cbAnalogConfig.Visible = True
        cbDin0_3.Visible = False
        cbRate.Visible = False
        cbCount.Visible = False
        'diplay the configuration of analog channel 3 in combo box cbChannelConfig
        If rbAin3.Checked Then
            cbAnalogConfig.Text = ChannelConfig(2)
        End If
    End Sub

    Private Sub rbAin4_CheckedChanged(sender As System.Object, e As System.EventArgs) Handles rbAin4.CheckedChanged
        'display the proper combo box
        cbAnalogConfig.Visible = True
        cbDin0_3.Visible = False
        cbRate.Visible = False
        cbCount.Visible = False
        'diplay the configuration of analog channel 4 in combo box cbChannelConfig
        If rbAin4.Checked Then
            cbAnalogConfig.Text = ChannelConfig(3)
        End If
    End Sub

    Private Sub rbAin5_CheckedChanged(sender As System.Object, e As System.EventArgs) Handles rbAin5.CheckedChanged
        'display the proper combo box
        cbAnalogConfig.Visible = True
        cbDin0_3.Visible = False
        cbRate.Visible = False
        cbCount.Visible = False
        'diplay the configuration of analog channel 5 in combo box cbChannelConfig
        If rbAin5.Checked Then
            cbAnalogConfig.Text = ChannelConfig(4)
        End If
    End Sub

    Private Sub rbAin6_CheckedChanged(sender As System.Object, e As System.EventArgs) Handles rbAin6.CheckedChanged
        'display the proper combo box
        cbAnalogConfig.Visible = True
        cbDin0_3.Visible = False
        cbRate.Visible = False
        cbCount.Visible = False
        'diplay the configuration of analog channel 6 in combo box cbChannelConfig
        If rbAin6.Checked Then
            cbAnalogConfig.Text = ChannelConfig(5)
        End If
    End Sub

    Private Sub rbAin7_CheckedChanged(sender As System.Object, e As System.EventArgs) Handles rbAin7.CheckedChanged
        'display the proper combo box
        cbAnalogConfig.Visible = True
        cbDin0_3.Visible = False
        cbRate.Visible = False
        cbCount.Visible = False
        'diplay the configuration of analog channel 7 in combo box cbChannelConfig
        If rbAin7.Checked Then
            cbAnalogConfig.Text = ChannelConfig(6)
        End If
    End Sub

    Private Sub rbAin8_CheckedChanged(sender As System.Object, e As System.EventArgs) Handles rbAin8.CheckedChanged
        'display the proper combo box
        cbAnalogConfig.Visible = True
        cbDin0_3.Visible = False
        cbRate.Visible = False
        cbCount.Visible = False
        'diplay the configuration of analog channel 8 in combo box cbChannelConfig
        If rbAin8.Checked Then
            cbAnalogConfig.Text = ChannelConfig(7)
        End If
    End Sub

    Private Sub btnClear_Click(sender As System.Object, e As System.EventArgs) Handles btnClear.Click
        'Clear the output list box
        lbOutput.Items.Clear()
        lblRealTime.Text = ""
    End Sub

    Private Sub cbDin0_1_SelectedIndexChanged(sender As System.Object, e As System.EventArgs) Handles cbDin0_3.SelectedIndexChanged
        'When combobox cbDin0_3 changes, update array ChannelConfig accordingly
        ChannelConfig(8) = cbDin0_3.Text
    End Sub

    Private Sub rbDin0_1_CheckedChanged(sender As System.Object, e As System.EventArgs) Handles rbDin0_1.CheckedChanged
        'display the proper combo boxes
        cbAnalogConfig.Visible = False
        cbDin0_3.Visible = True
        cbRate.Visible = False
        cbCount.Visible = False
        'diplay the configuration of analog channel 4 in combo box cbChannelConfig
        cbDin0_3.Text = ChannelConfig(8)
    End Sub

    Private Sub rbDin2_CheckedChanged(sender As System.Object, e As System.EventArgs) Handles rbRate.CheckedChanged
        'display the proper combo boxes
        cbAnalogConfig.Visible = False
        cbDin0_3.Visible = False
        cbRate.Visible = True
        cbCount.Visible = False
        'diplay the configuration of analog channel 4 in combo box cbChannelConfig
        cbRate.Text = ChannelConfig(9)
    End Sub

    Private Sub rbDin3_CheckedChanged(sender As System.Object, e As System.EventArgs) Handles rbCount.CheckedChanged
        'display the proper combo boxes
        cbAnalogConfig.Visible = False
        cbDin0_3.Visible = False
        cbRate.Visible = False
        cbCount.Visible = True
        'diplay the configuration of analog channel 4 in combo box cbChannelConfig
        cbCount.Text = ChannelConfig(10)
    End Sub

    Private Sub cbDin2_SelectedIndexChanged(sender As System.Object, e As System.EventArgs) Handles cbRate.SelectedIndexChanged
        ChannelConfig(9) = cbRate.Text
    End Sub

    Private Sub cbDin3_SelectedIndexChanged(sender As System.Object, e As System.EventArgs) Handles cbCount.SelectedIndexChanged
        ChannelConfig(10) = cbCount.Text
    End Sub

    Private Sub ckbDout0_CheckedChanged(sender As System.Object, e As System.EventArgs) Handles ckbDout0.CheckedChanged
        SetDigitalOutputs()
    End Sub

    Private Sub ckbDout1_CheckedChanged(sender As System.Object, e As System.EventArgs) Handles ckbDout1.CheckedChanged
        SetDigitalOutputs()
    End Sub

    Private Sub ckbDout2_CheckedChanged(sender As System.Object, e As System.EventArgs) Handles ckbDout2.CheckedChanged
        SetDigitalOutputs()
    End Sub

    Private Sub ckbDout3_CheckedChanged(sender As System.Object, e As System.EventArgs) Handles ckbDout3.CheckedChanged
        SetDigitalOutputs()
    End Sub
End Class
